﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Controls.GridView;
using Telerik.Windows.Controls;
using System.ComponentModel;

namespace WpfApplication16
{
    public class ReorderingEventArgs : CancelEventArgs
    {
        public GridViewDataControl SourceGrid
        {
            get;
            private set;
        }
        public IEnumerable<object> DraggedItems
        {
            get;
            private set;
        }
        public int DropIndex
        {
            get;
            private set;
        }
        public ReorderingEventArgs(GridViewDataControl sourceGrid, IEnumerable<object> draggedItems, int dropIndex)
            : base(false)
        {
            this.SourceGrid = sourceGrid;
            this.DraggedItems = draggedItems;
            this.DropIndex = dropIndex;
        }
    }

    public class ReorderedEventArgs : EventArgs
    {
        public GridViewDataControl SourceGrid
        {
            get;
            private set;
        }
        public IEnumerable<object> DraggedItems
        {
            get;
            private set;
        }
        public ReorderedEventArgs(GridViewDataControl sourceGrid, IEnumerable<object> draggedItems)
            : base()
        {
            this.SourceGrid = sourceGrid;
            this.DraggedItems = draggedItems;
        }
    }

    public class DragStartedEventArgs : EventArgs
    {
        public RadGridView SourceGrid
        {
            get;
            private set;
        }
        public IEnumerable<object> DraggedItems
        {
            get;
            private set;
        }
        public DragStartedEventArgs(RadGridView sourceGrid, IEnumerable<object> draggedItems)
            : base()
        {
            this.SourceGrid = sourceGrid;
            this.DraggedItems = draggedItems;
        }
    }

    public class BeginningDragEventArgs : CancelEventArgs
    {
        public RadGridView SourceGrid
        {
            get;
            private set;
        }
        public IEnumerable<object> DraggedItems
        {
            get;
            private set;
        }
        public BeginningDragEventArgs(RadGridView sourceGrid, IEnumerable<object> draggedItems)
            : base()
        {
            this.SourceGrid = sourceGrid;
            this.DraggedItems = draggedItems;
        }
    }

    public static class GridViewDataControlExtensions
    {
        internal static RadRowItem GetContainerFromItemRecursive(this BaseItemsControl itemsControl, object dataItem)
        {
            RadRowItem container = null;
            var panel = itemsControl.ChildrenOfType<GridViewVirtualizingPanel>().FirstOrDefault();
            if (itemsControl != null && panel != null)
            {
                foreach (var child in panel.Children)
                {
                    var groupRow = child as GridViewGroupRow;
                    var row = child as GridViewRow;
                    if (groupRow != null)
                        container = groupRow.GetContainerFromItemRecursive(dataItem);
                    if (row != null && ReferenceEquals(row.Item, dataItem))
                        container = child as RadRowItem;
                    if (container != null)
                        return container;
                }
            }
            return container;
        }
        internal static RadRowItem GetContainerFromDataItem(this GridViewDataControl dataControl, object item)
        {
            if (item == null || dataControl == null)
                return null;
            if (dataControl.IsGrouping)
                return dataControl.GetContainerFromItemRecursive(item);
            else
                return dataControl.ItemContainerGenerator.ContainerFromItem(item) as RadRowItem;
        }
    }
}
